/*
 * Copyright © 2021, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.administer;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import net.javacomm.client.resource.Resource;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.MultilingualButton;
import net.javacomm.multilingual.MultilingualLabel;
import net.javacomm.multilingual.MultilingualString;
import net.javacomm.multilingual.schema.ISO639;
import net.javacomm.multilingual.schema.KEY;
import net.javacomm.window.manager.Control;



@SuppressWarnings("serial")
public class JMailserver extends JPanel implements Babelfish {

  private final int textfield_len = 20;
  private GridBagLayout gridbag = new GridBagLayout();
  private GridBagConstraints con = new GridBagConstraints();
  private MultilingualLabel labelSmtpUser = new MultilingualLabel(KEY.LABEL_BENUTZERNAME_POSTAUSGANG);
  private JTextField textfieldSmtpUser = new JTextField(textfield_len);
  private MultilingualLabel labelSmtpHost = new MultilingualLabel(KEY.LABEL_POSTAUSGANGSERVER_MAILVERSAND);
  private JTextField textfieldSmtpHost = new JTextField(textfield_len);

  private MultilingualString ueberDenMailserver = new MultilingualString(KEY.STRING_UEBER_DEN_MAILSERVER);

  private MultilingualLabel labelPort = new MultilingualLabel(KEY.LABEL_PORT_AUSGANGSSERVER);
  private JTextField textfieldPort = new JTextField(4);

  private MultilingualLabel labelPasswort = new MultilingualLabel(KEY.LABEL_PASSWORT_POSTAUSGANGSERVER);
  private JTextField textfieldPasswort = new JTextField(textfield_len);
  private MultilingualLabel labelAdministratorPostfach = new MultilingualLabel(
      KEY.LABEL_ADMINISTRATORPOSTFACH
  );
  private JTextField textfieldAdministratorPostfach = new JTextField(textfield_len);
  private JSeparator separator = new JSeparator();
  private MultilingualButton buttonSchliessen = new MultilingualButton(KEY.BUTTON_SCHLIESSEN);
  private MultilingualButton buttonSpeichern = new MultilingualButton(KEY.BUTTON_SPEICHERN);
  private MultilingualButton buttonTestmail = new MultilingualButton(KEY.BUTTON_TESTMAIL);
  private GridLayout grid = new GridLayout(1, 3, 12, 12);
  private JPanel buttonpane = new JPanel(grid);
  private PropertyChangeSupport changes = new PropertyChangeSupport(this);
  private final int textfield_left = 6;
  private final int margin_left = 12;
  private final int margin_right = 12;
  private final int margin_top = 12;
  private MultilingualLabel labelEinleitung = new MultilingualLabel();
  private FocusAdapter focusAdapter = new FocusAdapter() {
    @Override
    public void focusGained(FocusEvent event) {
      textfieldPort.setBackground(Color.WHITE);
      textfieldPort.setForeground(Color.BLACK);
    }
  };
  private MultilingualString contextXml = new MultilingualString(KEY.STRING_IST_IN_DER_CONTEXT);

  public JMailserver() {
    setLayout(gridbag);

    con.gridx = 0;
    con.gridy = 0;
    con.gridwidth = 2;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(margin_top, margin_left, 24, margin_right);
    con.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(labelEinleitung, con);
    add(labelEinleitung);

    con.gridx = 0;
    con.gridy = 1;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(0, margin_left, 0, 0);
    con.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(labelSmtpUser, con);
    add(labelSmtpUser);

    con.gridx = 1;
    con.gridy = 1;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(0, textfield_left, 0, margin_right);
    con.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(textfieldSmtpUser, con);
    add(textfieldSmtpUser);

    con.gridx = 0;
    con.gridy = 2;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, margin_left, 0, 0);
    con.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(labelSmtpHost, con);
    add(labelSmtpHost);

    con.gridx = 1;
    con.gridy = 2;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, textfield_left, 0, margin_right);
    con.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(textfieldSmtpHost, con);
    add(textfieldSmtpHost);

    con.gridx = 0;
    con.gridy = 3;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, margin_left, 0, 0);
    con.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(labelPort, con);
    add(labelPort);

    con.gridx = 1;
    con.gridy = 3;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, textfield_left, 0, margin_right);
    con.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(textfieldPort, con);
    add(textfieldPort);
    textfieldPort.addFocusListener(focusAdapter);

    con.gridx = 0;
    con.gridy = 4;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, margin_left, 0, 0);
    con.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(labelPasswort, con);
    add(labelPasswort);

    con.gridx = 1;
    con.gridy = 4;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, textfield_left, 0, margin_right);
    con.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(textfieldPasswort, con);
    add(textfieldPasswort);

    con.gridx = 0;
    con.gridy = 5;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, margin_left, 0, 0);
    con.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(labelAdministratorPostfach, con);
    add(labelAdministratorPostfach);

    con.gridx = 1;
    con.gridy = 5;
    con.gridwidth = 1;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, textfield_left, 0, margin_right);
    con.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(textfieldAdministratorPostfach, con);
    add(textfieldAdministratorPostfach);

    con.gridx = 0;
    con.gridy = 6;
    con.gridwidth = 2;
    con.gridheight = 1;
    con.fill = GridBagConstraints.HORIZONTAL;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(24, margin_left, 0, margin_right);
    con.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(separator, con);
    add(separator);

    buttonpane.add(buttonTestmail);
    buttonpane.add(buttonSpeichern);
    buttonpane.add(buttonSchliessen);
    con.gridx = 0;
    con.gridy = 7;
    con.gridwidth = 2;
    con.gridheight = 1;
    con.fill = GridBagConstraints.NONE;
    con.weightx = 0.0;
    con.weighty = 0.0;
    con.insets = new Insets(12, margin_left, 12, margin_right);
    con.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(buttonpane, con);
    add(buttonpane);

    buttonTestmail.addActionListener((event) -> {
      changes.firePropertyChange(JMailserver.class.getName(), Control.NULL, Control.TESTMAIL);
    });

    buttonSchliessen.addActionListener((event) -> {
      changes.firePropertyChange(JMailserver.class.getName(), Control.NULL, Control.CLOSE);
    });

    buttonSpeichern.addActionListener((event) -> {
      changes.firePropertyChange(JMailserver.class.getName(), Control.NULL, Control.SAVE);
    });

  }



  public synchronized void addMailserverListener(PropertyChangeListener listener) {
    changes.addPropertyChangeListener(listener);
  }



  public synchronized void removeAllListener() {
    for (PropertyChangeListener listener : changes.getPropertyChangeListeners()) {
      removeMailserverListener(listener);
    }
    textfieldPort.removeFocusListener(focusAdapter);
    for (ActionListener al : buttonSchliessen.getActionListeners()) {
      buttonSchliessen.removeActionListener(al);
    }
    for (ActionListener al : buttonTestmail.getActionListeners()) {
      buttonTestmail.removeActionListener(al);
    }
    for (ActionListener al : buttonSpeichern.getActionListeners()) {
      buttonSpeichern.removeActionListener(al);
    }
  }



  public synchronized void removeMailserverListener(PropertyChangeListener listener) {
    changes.removePropertyChangeListener(listener);
  }



  /**
   * Legt die Postfachadresse fest.
   * 
   * @param value
   *              ein Postafach
   */
  public void setAdministratorPostfach(String value) {
    textfieldAdministratorPostfach.setText(value);
  }



  public String getAdministratorPostfach() {
    return textfieldAdministratorPostfach.getText();
  }



  /**
   * Legt den Benutzernamen für den Mailserver fest.
   * 
   * @param value
   *              der Benutzername auf dem Postausgangsserver
   */
  public void setBenutzername(String value) {
    textfieldSmtpUser.setText(value);
  }



  public String getBenutzername() {
    return textfieldSmtpUser.getText();
  }



  /**
   * Der Hostname für den Mailserver.
   * 
   * @param value
   *              der Postausgangsserver
   */
  public void setHost(String value) {
    textfieldSmtpHost.setText(value);
  }



  public String getHost() {
    return textfieldSmtpHost.getText();
  }



  public void setPort(int port) {
    textfieldPort.setText(String.valueOf(port));
  }



  /**
   * Der SMTP-Port wird zurückgegeben.
   * 
   * 
   * @return der Port
   * @throws StringException
   *                         der Rückgabewert ist ein String
   */
  public int getPort() throws StringException {
    String port = textfieldPort.getText();
    Integer value;
    try {
      value = Integer.valueOf(port);
    }
    catch (NumberFormatException e) {
      textfieldPort.setBackground(Resource.JQUERY_RED);
      textfieldPort.setForeground(Color.WHITE);
      throw new StringException(port);
    }
    return value;
  }



  /**
   * Das Passwort für den Mailaccount.
   * 
   * @param value
   *              ein Passwort
   * 
   * @deprecated das Passwort ist auf dem Server in der context.xml hinterlegt.
   */
  @Deprecated
  final void setPasswort(String value) {
    textfieldPasswort.setText(value);
  }



  public char[] getPasswort() {
//    return textfieldPasswort.getPassword();
    return textfieldPasswort.getText().toCharArray();
  }



  @Override
  public void setLanguage(ISO639 code) {
    ueberDenMailserver.setLanguage(code);
    labelEinleitung.setText("<html>" + "<p>" + ueberDenMailserver.toString() + "</p>" + "</html>");
    labelPort.setLanguage(code);
    labelSmtpUser.setLanguage(code);
    labelSmtpHost.setLanguage(code);
    labelPasswort.setLanguage(code);
    labelAdministratorPostfach.setLanguage(code);
    buttonSpeichern.setLanguage(code);
    buttonSchliessen.setLanguage(code);
    buttonTestmail.setLanguage(code);
    contextXml.setLanguage(code);
    textfieldAdministratorPostfach.setText(contextXml.toString());
    textfieldPasswort.setText(contextXml.toString());
  }

}
