/*
 * Copyright © 2025, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.phone;

import javax.sound.sampled.LineUnavailableException;



/**
 * Diese Klasse wird für Telefonkonferenzen verwendet. Für jeden
 * Konferenzteilnehmer muss eine separate Lautsprecherleitung eingerichtet
 * werden.
 */
public class ConferenceSpeaker extends Speaker {

  private String userid;

  /**
   * Der Lautsprecher wird für einen Absender eingerichtet. Für jeden Absender
   * wird eine Mixerleitung geöffnet. Über die {@code userid} wird die Leitung
   * identifiziert.
   * 
   * @param speakername
   *                      dieser Lautsprechername
   * @param telefonformat
   *                      dieses Telefonformat
   * @param userid
   *                      diese Userid
   * 
   * @throws MixerNotFoundException
   *                                  für diesen Lautsprecher konnte kein Mixer
   *                                  gefunden werden
   * @throws LineUnavailableException
   *                                  diese Leitung ist belegt oder wird von einer
   *                                  anderen Anwendung benutzt
   */
  public ConferenceSpeaker(String speakername, Telefonformat telefonformat, String userid)
      throws MixerNotFoundException, LineUnavailableException {
    super(speakername, telefonformat);
    this.userid = userid;
  }



  public String getUserid() {
    return userid;
  }

}
