/*
 * Copyright © 2020, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.client.phone;

import java.util.EventObject;


/**
 * Alle Ereignisse auf einem UDP-Port.
 *  
 * @author llange
 *
 */
public class UDPEvent extends EventObject {

  private static final long serialVersionUID = 5571965069425296679L;
  private byte[] data;
  
  
  /**
   * Der Kommunikationsendpunkt meldet eingehende Daten.
   * 
   * @param source
   *               das auslösende Objekt
   */
  public UDPEvent(Object source) {
    super(source);
  }

  

  /**
   * Der Kommunikationsendpunkt meldet eingehende Daten.
   * 
   * 
   * @param source
   *               das auslösende Objekt
   * @param data
   *               eine Datenkolonne aus dem Netzwerk
   */
  public UDPEvent(Object source, byte[] data) {
    super(source);
    setData(data);
  }
  
  
  /**
   * Setze eine Datenkolonne
   * 
   * @param content eine Datenkolonne
   */
  public void setData(byte[] content) {
    this.data = content;
  }
  
  
  
  /**
   * Gib den eingegangenen Inhalt zurück.
   * 
   * @return eine Datenkolonne aus dem Netzwerk
   */
  public byte[] getData() {
    return data;
  }
  
  
}
