package net.javacomm.restserver;

import static net.javacomm.protocol.Protocol.KONFERENZRAUM;
import static net.javacomm.protocol.Protocol.ORGANISATOR;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.WebApplicationException;
import jakarta.ws.rs.core.MediaType;
import java.io.IOException;
import java.io.InputStream;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.glassfish.jersey.media.multipart.FormDataBodyPart;
import org.glassfish.jersey.media.multipart.FormDataMultiPart;
import net.javacomm.database.DatabaseException;
import net.javacomm.database.WebdatabaseImpl;
import net.javacomm.protocol.Protocol;



/**
 * Über diese Klasse werden die Bilder einer Vidokonferenz in die Tabelle
 * {@code TB_TELKO} geschriebn und gelesen.
 */
@Path("/videokonferenz")
public class Videokonferenz {

  final static Logger log = LogManager.getLogger(Videokonferenz.class);
  WebdatabaseImpl database;

  public Videokonferenz() {
    database = (WebdatabaseImpl) WebdatabaseImpl.getInstance();
  }



  @POST
  @Path("/upload")
  public boolean uploadScreenshot(FormDataMultiPart multiPart) {
    boolean done = false;
    FormDataBodyPart konferenzraum = multiPart.getField(Protocol.KONFERENZRAUM);
    FormDataBodyPart organisator = multiPart.getField(Protocol.ORGANISATOR);
    FormDataBodyPart content = multiPart.getField(Protocol.SCREENSHOT);

    try(InputStream is = content.getContent()) {
      done = database.updateVideokonferenzScreenshot(konferenzraum.getValue(), organisator.getValue(), is);
    }
    catch (DatabaseException | IOException e) {
      log.error(e.getMessage());
      throw new WebApplicationException();
    }
    return done;
  }



  /**
   * Lies ein Bild für die Videokonferenz aus.
   * 
   * @param konferenzraum
   *                      dieser Konferenzraum
   * @param organisator
   *                      dieser Anwender hat die Konferenz eingerichtet
   * @return dieser Stream verweist auf die lerzte Bilddatei; null ist erlaubt,
   *         wenn kein Bild in der Datenbank vorliegt
   */
  @GET
  @Path("/download/{" + Protocol.KONFERENZRAUM + "}/{" + ORGANISATOR + "}")
  @Produces(MediaType.APPLICATION_OCTET_STREAM)
  public InputStream downloadScreenshot(@PathParam(value = KONFERENZRAUM) String konferenzraum,
      @PathParam(value = ORGANISATOR) String organisator) {

    InputStream stream;
    try {
      stream = database.downloadVideokonferenzScreenshot(konferenzraum, organisator);
    }
    catch (DatabaseException e) {
      log.error(e.getMessage());
      throw new WebApplicationException();
    }
    return stream;

  }

}
