package net.javacomm.server;

import java.util.List;



/**
 * Der TelkoBroadcaster sammelt alle Sessions einer Telko. An diese Sessions
 * wird gesendet. Der Server kennt alle TelkoClients. Der Server merkt sich nur
 * die aktiven Clients einer Konferenz. Diese Anzahl ist variabel.
 */
public class TelkoClient {

  private String organisator;
  private String konferenzraum;
  private List<String> sessions;

  /**
   * Der Konstruktor ist ein zusammengesetzter Schlüssel aus
   * (Organisator/Konferenzraum). Dieses Schlüsselpaar ist unveränderbar.
   * 
   * @param organisator
   *                      dieser Anwender hat die Konferenz eingerichtet
   * @param konferenzraum
   *                      dieser Konferenzraum
   */
  public TelkoClient(String organisator, String konferenzraum) {
    this.organisator = organisator;
    this.konferenzraum = konferenzraum;
  }



  /**
   * Wer hat diesen Konferenzraum eingerichtet?
   * 
   * @return dieser Organisator
   */
  public String getOrganisator() {
    return organisator;
  }



  /**
   * Wie heißt der Konferenzraum?
   * 
   * @return dieser Konferenzraum
   */
  public String getKonferenzraum() {
    return konferenzraum;
  }



  /**
   * 
   * @return
   */
  public List<String> getSessions() {
    return sessions;
  }



  /**
   * An dieses Sessions muss gesendet werden.
   * 
   * @param sessions
   *                 alle Empfänger
   */
  public void setSessions(List<String> sessions) {
    this.sessions = sessions;
  }

}
