/*
 * Copyright © 2020, Luis Andrés Lange <https://javacomm.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
package net.javacomm.mastermind.audio;

import com.github.trilarion.sound.vorbis.sampled.spi.VorbisAudioFileReader;
import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import net.javacomm.mastermind.gui.Constants;



/**
 * SoundClip spielt asynchron Vorbis-Dateien ab.
 * 
 */
public final class SoundClip {

  private static Clip jumpingInVideogameClip;
  private static Clip gamersFailClip;
  private static Clip simpleGameCountdownClip;
  private static Clip whistleGameClip;
  private static Clip clickErrorClip;
  private static Clip playerRehargingClip;
  private static Clip completionOfALevelClip;
  private static Clip electricClip;
  private static Clip victoryClip;
  private static Clip cartoonGossipClip;
  private static Clip maleTiredYawnClip;
  private static Clip longGameoverClip;
  private static Clip monkeyApplauseClip;
  private static Class<? extends SoundClip> resource;
  private static VorbisAudioFileReader jumpingInVideogameReader;
  private static VorbisAudioFileReader gamersFailReader;
  private static VorbisAudioFileReader simpleGameCountdownReader;
  private static VorbisAudioFileReader whistleGameReader;
  private static VorbisAudioFileReader clickErrorReader;
  private static VorbisAudioFileReader playerRehargingReader;
  private static VorbisAudioFileReader playerCompletionOfALevelReader;
  private static VorbisAudioFileReader playerElectricReader;
  private static VorbisAudioFileReader playerVictoryReader;
  private static VorbisAudioFileReader playerCartoonGossipReader;
  private static VorbisAudioFileReader playerMaleTiredYawnReader;
  private static VorbisAudioFileReader playerLongGameoverReader;
  private static VorbisAudioFileReader playerMonkeyApplauseReader;

  static {
    try {
      new SoundClip();
    }
    catch (LineUnavailableException e) {
      throw new RuntimeException(e);
    }
  }

  private SoundClip() throws LineUnavailableException {
    resource = getClass();
    jumpingInVideogameClip = AudioSystem.getClip();
    gamersFailClip = AudioSystem.getClip();
    simpleGameCountdownClip = AudioSystem.getClip();
    whistleGameClip = AudioSystem.getClip();
    clickErrorClip = AudioSystem.getClip();
    playerRehargingClip = AudioSystem.getClip();
    completionOfALevelClip = AudioSystem.getClip();
    electricClip = AudioSystem.getClip();
    victoryClip = AudioSystem.getClip();
    cartoonGossipClip = AudioSystem.getClip();
    maleTiredYawnClip = AudioSystem.getClip();
    longGameoverClip = AudioSystem.getClip();
    monkeyApplauseClip = AudioSystem.getClip();
  }



  public static void playMonkeyApplause()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!monkeyApplauseClip.isOpen()) {
      playerMonkeyApplauseReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerMonkeyApplauseReader
            .getAudioInputStream(resource.getResource(Constants.MONKEY_APPLAUSE));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          monkeyApplauseClip.open(targetStream);
        }
      }
    }
    monkeyApplauseClip.setFramePosition(0);
    monkeyApplauseClip.start();
  }



  public static void stopMonkeyApplauseClip() {
    monkeyApplauseClip.stop();
    monkeyApplauseClip.close();
  }

//////////////////////////////////////////////////////



  public static void playLongGameover()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!longGameoverClip.isOpen()) {
      playerLongGameoverReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerLongGameoverReader
            .getAudioInputStream(resource.getResource(Constants.LONG_GAME_OEVER));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          longGameoverClip.open(targetStream);
        }
      }
    }
    longGameoverClip.setFramePosition(0);
    longGameoverClip.start();
  }



  public static void stopLongGameoverClip() {
    longGameoverClip.stop();
    longGameoverClip.close();
  }

//////////////////////////////////////////////////////



  public static void playMaleTiredYawn()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!maleTiredYawnClip.isOpen()) {
      playerMaleTiredYawnReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerMaleTiredYawnReader
            .getAudioInputStream(resource.getResource(Constants.MALE_TIRED_YAWN));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          maleTiredYawnClip.open(targetStream);
        }
      }
    }
    maleTiredYawnClip.setFramePosition(0);
    maleTiredYawnClip.start();
  }



  public static void stopMaleTiredYawnClip() {
    maleTiredYawnClip.stop();
    maleTiredYawnClip.close();
  }

//////////////////////////////////////////////////////



  public static void playCartoonGossip()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!cartoonGossipClip.isOpen()) {
      playerCartoonGossipReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerCartoonGossipReader
            .getAudioInputStream(resource.getResource(Constants.CARTOON_GOSSIP));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          cartoonGossipClip.open(targetStream);
        }
      }
    }
    cartoonGossipClip.setFramePosition(0);
    cartoonGossipClip.start();
  }



  public static void stopCartoonGossipClip() {
    cartoonGossipClip.stop();
    cartoonGossipClip.close();
  }

//////////////////////////////////////////////////////



  public static void playVictory()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!victoryClip.isOpen()) {
      playerVictoryReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerVictoryReader
            .getAudioInputStream(resource.getResource(Constants.VICTORY));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          victoryClip.open(targetStream);
        }
      }
    }
    victoryClip.setFramePosition(0);
    victoryClip.start();
  }



  public static void stopVictory() {
    victoryClip.stop();
    victoryClip.close();
  }

//////////////////////////////////////////////////////



  public static void playElectric()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!electricClip.isOpen()) {
      playerElectricReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerElectricReader
            .getAudioInputStream(resource.getResource(Constants.ELECTRIC));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          electricClip.open(targetStream);
        }
      }
    }
    electricClip.setFramePosition(0);
    electricClip.start();
  }



  public static void stopElectric() {
    electricClip.stop();
    electricClip.close();
  }

//////////////////////////////////////////////////////



  public static void playCompletionOfALevel()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!completionOfALevelClip.isOpen()) {
      playerCompletionOfALevelReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerCompletionOfALevelReader
            .getAudioInputStream(resource.getResource(Constants.COMPLETION_OF_A_LEVEL));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          completionOfALevelClip.open(targetStream);
        }
      }
    }
    completionOfALevelClip.setFramePosition(0);
    completionOfALevelClip.start();
  }



  public static void stopCompletionOfALevel() {
    completionOfALevelClip.stop();
    completionOfALevelClip.close();
  }

//////////////////////////////////////////////////////



  public static void playPlayerRecharging()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!playerRehargingClip.isOpen()) {
      playerRehargingReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = playerRehargingReader
            .getAudioInputStream(resource.getResource(Constants.PLAYER_RECHARGING));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          playerRehargingClip.open(targetStream);
        }
      }
    }
    playerRehargingClip.setFramePosition(0);
    playerRehargingClip.start();
  }



  public static void stopPlayerReharging() {
    playerRehargingClip.stop();
    playerRehargingClip.close();
  }

//////////////////////////////////////////////////////



  public static void playClickError()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!clickErrorClip.isOpen()) {
      clickErrorReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = clickErrorReader
            .getAudioInputStream(resource.getResource(Constants.CLICK_ERROR));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          clickErrorClip.open(targetStream);
        }
      }
    }
    clickErrorClip.setFramePosition(0);
    clickErrorClip.start();
  }



  public static void stopClickError() {
    clickErrorClip.stop();
    clickErrorClip.close();
  }

//////////////////////////////////////////////////////



  public static void playWhistleGame()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!whistleGameClip.isOpen()) {
      whistleGameReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = whistleGameReader
            .getAudioInputStream(resource.getResource(Constants.WHISTLE_GAME));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          whistleGameClip.open(targetStream);
        }
      }
    }
    whistleGameClip.setFramePosition(0);
    whistleGameClip.start();
  }



  public static void stopWhistleGame() {
    whistleGameClip.stop();
    whistleGameClip.close();
  }

//////////////////////////////////////////////////////



  public static void playSimpleGameCountdown()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!simpleGameCountdownClip.isOpen()) {
      simpleGameCountdownReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = simpleGameCountdownReader
            .getAudioInputStream(resource.getResource(Constants.SIMPLE_GAME_COUNTDOWN));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          simpleGameCountdownClip.open(targetStream);
        }
      }
    }
    simpleGameCountdownClip.setFramePosition(0);
    simpleGameCountdownClip.start();
  }



  public static void stopSimpleGameCountdown() {
    simpleGameCountdownClip.stop();
    simpleGameCountdownClip.close();
  }

//////////////////////////////////////////////////////



  public static void playGamersFail()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!gamersFailClip.isOpen()) {
      gamersFailReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = gamersFailReader
            .getAudioInputStream(resource.getResource(Constants.GAMERS_FAIL));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          gamersFailClip.open(targetStream);
        }
      }
    }
    gamersFailClip.setFramePosition(0);
    gamersFailClip.start();
  }



  public static void stopGamersFail() {
    gamersFailClip.stop();
    gamersFailClip.close();
  }

//////////////////////////////////////////////////////



  public static void playJumpingInVideogame()
      throws IOException, UnsupportedAudioFileException, LineUnavailableException {
    if (!jumpingInVideogameClip.isOpen()) {
      jumpingInVideogameReader = new VorbisAudioFileReader();
      try(
        AudioInputStream oggStream = jumpingInVideogameReader
            .getAudioInputStream(resource.getResource(Constants.JUMPING_IN_A_VIDEOGAME));
        AudioInputStream audiostream = AudioSystem.getAudioInputStream(oggStream)
      ) {

        AudioFormat ringtoneFormat = oggStream.getFormat();
        AudioFormat pcmFormat = new AudioFormat(
            AudioFormat.Encoding.PCM_SIGNED, ringtoneFormat.getSampleRate(), 16, ringtoneFormat.getChannels(),
            ringtoneFormat.getChannels() * 2, ringtoneFormat.getSampleRate(), false
        );

        // konvertiert
        try(AudioInputStream targetStream = AudioSystem.getAudioInputStream(pcmFormat, oggStream);) {
          jumpingInVideogameClip.open(targetStream);
        }
      }
    }
    jumpingInVideogameClip.setFramePosition(0);
    jumpingInVideogameClip.start();
  }



  /**
   * Der JumpingInAVideogame wird abgestellt.
   */
  public static void stopJumpingInVideogame() {
    jumpingInVideogameClip.stop();
    jumpingInVideogameClip.close();
  }



  /**
   * Die Resourcen aller Soundclips werden freigegeben.
   * 
   */
  public static void release() {
    jumpingInVideogameClip.close();
    gamersFailClip.close();
    simpleGameCountdownClip.close();
    whistleGameClip.close();
    clickErrorClip.close();
    playerRehargingClip.close();
    completionOfALevelClip.close();
    electricClip.close();
    victoryClip.close();
    cartoonGossipClip.close();
    maleTiredYawnClip.close();
    longGameoverClip.close();
    monkeyApplauseClip.close();
  }

}
