/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.control;

/**
 * Wird geworfen, wenn in Mastermind ein nicht vorhergesehener Spielfehler
 * auftrat, der das weitere Spiel unmöglich macht. Unmöglich meint im Sinne
 * undefinierter Zustand, der nicht den Spielregeln entspricht.
 */
public class MastermindException extends Exception {

  /**
   * Basisklasse aller Exceptions im Mastermindprogramm
   */
  private static final long serialVersionUID = 5687928287797504460L;

  public MastermindException() {
    super();
  }



  /**
   * Mastermind hat einen Spielfehler.
   * 
   * @param message
   *                Fehlermeldung
   */
  public MastermindException(String message) {
    super(message);
  }



  /**
   * Mastermind hat einen Spielfehler.
   * 
   * @param cause
   *              Fehlerursache
   */
  public MastermindException(Throwable cause) {
    super(cause);
  }



  /**
   * Mastermind hat einen Spielfehler.
   * 
   * @param message
   *                Fehlermeldung
   * @param cause
   *                Fehlerursache
   */
  public MastermindException(String message, Throwable cause) {
    super(message, cause);
  }

}
