/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.control;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;



/**
 * Der {@code StatusController} informiert Interessenten über Statusänderungen.
 */
public class StatusController {

  private List<StatusListener> statusList = Collections.synchronizedList(new LinkedList<StatusListener>());

  public StatusController() {}



  /**
   * Interessenten können sich registrieren, um Statusänderungen mitgeteilt zu
   * bekommen.
   * 
   * @param listener
   *                 ein Interessent
   */
  public void addStatusListener(StatusListener listener) {
    statusList.add(listener);
  }



  /**
   * Ein Interessent deregistriert sich, um keine Änderungen zugestellt zu
   * bekommen.
   * 
   * @param listener
   *                 ein Interessent
   */
  public void removeStatusListener(StatusListener listener) {
    statusList.remove(listener);
  }



  public void removeAllListener() {
    statusList.clear();
  }



  /**
   * Diese Methode löst das Ereignis aus.
   *
   *
   * @param event
   *              das ausgelöste Ereignis
   */
  public void trigger(StatusEvent event) {
    for (StatusListener listener : statusList) {
      listener.stateChanged(event);
    }
  }

}
