/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D.Double;
import net.javacomm.mastermind.tool.DefColors;



@SuppressWarnings("serial")
public class EllipseFigure extends Double {

//  private final static Logger log = LogManager.getLogger(EllipseFigure.class);

  private DefColors blackBrownWhite;
  private Stroke stroke = new BasicStroke(5.0f);
  private int x = 0;
  private int y = 0;

  public EllipseFigure(int x, int y) {
    this.x = x;
    this.y = y;
    setColor(DefColors.BLACK);
  }



  public void setColor(DefColors value) {
    switch(value) {
      case BLACK:
        blackBrownWhite = DefColors.BLACK;
        break;
      case BROWN:
        blackBrownWhite = DefColors.BROWN;
        break;
      case WHITE:
        blackBrownWhite = DefColors.WHITE;
        break;
      default:
        throw new UnsupportedOperationException();
    }
  }



  public void setPosition(int x, int y) {
    this.x = x;
    this.y = y;
  }



  public void paintGraphic(Graphics2D g) {
    g.setStroke(stroke);
    switch(blackBrownWhite) {
      case BLACK:
        g.setColor(DefColors.BLACK.colorValue());
        g.fillOval(x + 15, y + 15, 15, 15);
        break;
      case BROWN:
        g.setColor(DefColors.BROWN.colorValue());
        g.fillOval(x, y, 45, 45);
        g.setColor(DefColors.BROWN.colorValue().brighter());
        g.drawOval(x, y, 45, 45);
        g.drawOval(x + 10, y + 10, 25, 25);
        g.drawOval(x + 20, y + 20, 5, 5);
        break;
      case WHITE:
        g.setColor(DefColors.WHITE.colorValue());
        g.fillOval(x, y, 45, 45);
        g.setColor(DefColors.WHITE.colorValue().darker());
        g.drawOval(x, y, 45, 45);
        g.drawOval(x + 10, y + 10, 25, 25);
        g.drawOval(x + 20, y + 20, 5, 5);
        break;
      default:
        throw new UnsupportedOperationException();
    }
  }

}
