/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.util.Hashtable;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import net.javacomm.mastermind.audio.SoundClip;
import net.javacomm.mastermind.control.Status;
import net.javacomm.mastermind.control.StatusEvent;
import net.javacomm.mastermind.control.StatusListener;
import net.javacomm.multilingual.Babelfish;
import net.javacomm.multilingual.MultilingualCheckBox;
import net.javacomm.multilingual.MultilingualLabel;
import net.javacomm.multilingual.MultilingualString;
import net.javacomm.multilingual.schema.ISO639;
import net.javacomm.multilingual.schema.KEY;
import net.javacomm.window.manager.Control;



/**
 * Über diese Klasse wird der Schwierigkeitsgrad eingestellt. In der
 * Grundstellung ist der Schwierigkeitsgrad 1.
 */
@SuppressWarnings("serial")
public class JDifficultyPanel extends JPanel implements Babelfish, StatusListener {

  private final static Logger log = LogManager.getLogger(JDifficultyPanel.class);

  private Color background = Constants.OLIV;
  private final static int MIN = 1;
  private int schwierigkeitsgrad = MIN;
  private JSlider regler = new JSlider(
      JSlider.VERTICAL, MIN, Constants.SCHWIERIGKEITSGRADE, schwierigkeitsgrad
  );
  private MultilingualLabel labelSchwiegrigkeitsgrad = new MultilingualLabel(KEY.LABEL_SCHWIERIGKEITSGRAD);
  private MultilingualLabel labMid = new MultilingualLabel(KEY.STRING_WEDER_NOCH);
  private MultilingualLabel labMin = new MultilingualLabel(KEY.LABEL_EINFACH);
  private MultilingualLabel labMax = new MultilingualLabel(KEY.LABEL_SCHWIERIG);
  private GridBagLayout gridbag = new GridBagLayout();
  private GridBagConstraints con = new GridBagConstraints();
  private Hashtable<Integer, JLabel> labelSlider = new Hashtable<Integer, JLabel>();
  private MultilingualCheckBox check = new MultilingualCheckBox(KEY.CHECKBOX_FARBMUSTER);
  private PropertyChangeSupport changes = new PropertyChangeSupport(this);
  private ImageIcon thinkman = new ImageIcon(getClass().getResource(Constants.THINKMAN));
  protected MultilingualLabel verloren = new MultilingualLabel(KEY.LABEL_DU_HAST_VERLOREN);
  private ImageIcon sadman = new ImageIcon(getClass().getResource(Constants.SADMAN));
  protected ImageIcon hurraman = new ImageIcon(getClass().getResource(Constants.HURRAMAN));
  protected MultilingualLabel gewonnen = new MultilingualLabel(KEY.LABEL_DU_HAST_GEWONNEN);

  private MultilingualString multilingualSchwierigkeitsgrad = new MultilingualString(
      KEY.LABEL_SCHWIERIGKEITSGRAD
  );
  private MultilingualString multilingualColoringByConditions = new MultilingualString(
      KEY.STRING_COLORING_BY_CONDITIONS
  );
  private MultilingualString multilingualColoringConditions = new MultilingualString(
      KEY.STRING_COLORING_CONDITIONS
  );

  private MultilingualString multilingualSchwierigkeitsgradAnpassen = new MultilingualString(
      KEY.STRING_DER_SCHWIERIGKEITSGRAD
  );
  private Status status;

  private ChangeListener changeListener = new ChangeListener() {

    @Override
    public void stateChanged(ChangeEvent e) {
      if (!regler.getValueIsAdjusting()) {

        switch(status) {
          case Init:
            schwierigkeitsgrad = regler.getValue();
            changes.firePropertyChange(JDifficultyPanel.class.getName(), Control.NULL, Control.SELECTED);
            break;
          case Lost:
            regler.removeChangeListener(changeListener);
            regler.setValue(schwierigkeitsgrad);

            gameOver();
            JOptionPane
                .showInternalMessageDialog(regler, verloren, "Game Over", JOptionPane.PLAIN_MESSAGE, sadman);
            regler.addChangeListener(changeListener);
            SoundClip.stopCartoonGossipClip();
            return;
          case Running:
            regler.removeChangeListener(changeListener);
            regler.setValue(schwierigkeitsgrad);

            errorClick();
            JOptionPane.showInternalMessageDialog(
                regler, multilingualSchwierigkeitsgradAnpassen.toString(),
                multilingualSchwierigkeitsgrad.toString(), JOptionPane.PLAIN_MESSAGE, thinkman
            );
            regler.addChangeListener(changeListener);
            return;

          case Won:
            regler.removeChangeListener(changeListener);
            regler.setValue(schwierigkeitsgrad);
            try {
              SoundClip.playVictory();
            }
            catch (IOException | UnsupportedAudioFileException | LineUnavailableException e1) {}
            JOptionPane.showInternalMessageDialog(
                regler, gewonnen, "Game Over", JOptionPane.PLAIN_MESSAGE, hurraman
            );
            regler.addChangeListener(changeListener);
            return;

          default:
            break;
        }

      }
    }

  };

  /**
   * Instanziiere die Klasse.
   */
  public JDifficultyPanel() {

    regler.setBackground(background);
    regler.setForeground(Constants.WHITE);
    regler.setPaintTicks(true);
    regler.setMinorTickSpacing(1);
    regler.addChangeListener(changeListener);

    labelSlider.put(1, labMin);
    labelSlider.put(2, labMid);
    labelSlider.put(3, labMax);
    regler.setLabelTable(labelSlider);
    regler.setPaintLabels(true);

    setBackground(background);

    setLayout(gridbag);

    con.gridx = 0;
    con.gridy = 0;
    con.insets = new Insets(0, 0, 0, 0);
    con.weightx = 0.0;
    gridbag.setConstraints(labelSchwiegrigkeitsgrad, con);
    add(labelSchwiegrigkeitsgrad);

    con.gridx = 0;
    con.gridy++;
    con.insets = new Insets(12, 0, 12, 0);
    con.weightx = 0.0;
    gridbag.setConstraints(regler, con);
    add(regler);

    con.gridx = 0;
    con.gridy++;
    con.insets = new Insets(0, 12, 0, 0);
    con.weightx = 0.0;
    gridbag.setConstraints(check, con);
    add(check);
    check.setIconTextGap(20);
    check.addActionListener(event -> {

      switch(status) {
        case Init:
          break;
        case Lost:
          check.setSelected(!check.isSelected());
          gameOver();
          JOptionPane
              .showInternalMessageDialog(check, verloren, "Game Over", JOptionPane.PLAIN_MESSAGE, sadman);
          SoundClip.stopCartoonGossipClip();
          break;
        case Running:
          check.setSelected(!check.isSelected());
          errorClick();
          JOptionPane.showInternalMessageDialog(
              check, multilingualColoringConditions.toString(), multilingualColoringByConditions.toString(),
              JOptionPane.PLAIN_MESSAGE, thinkman
          );

          break;
        case Won:
          check.setSelected(!check.isSelected());

          // du hast gewonnen und yeah, hurra stickman
          try {
            SoundClip.playVictory();
          }
          catch (IOException | UnsupportedAudioFileException | LineUnavailableException e1) {}
          JOptionPane
              .showInternalMessageDialog(regler, gewonnen, "Game Over", JOptionPane.PLAIN_MESSAGE, hurraman);

          break;
        default:
          break;

      }

    });
  }



  private void gameOver() {
    try {
      SoundClip.playCartoonGossip();
    }
    catch (IOException | UnsupportedAudioFileException | LineUnavailableException e) {}
  }



  private void errorClick() {
    try {
      SoundClip.playClickError();
    }
    catch (IOException | UnsupportedAudioFileException | LineUnavailableException e) {}
  }



  public int getLevel() {
    return schwierigkeitsgrad;
  }



  public boolean isSameColorsAreAllowed() {
    return check.isSelected();
  }



  /**
   * Wenn der Schwierigkeitsgrad sich ändert, wird ein Event ausgelöst.
   * 
   * @param listener
   */
  public void addDifficultListener(PropertyChangeListener listener) {
    changes.addPropertyChangeListener(listener);
  }



  public void removeDifficultListener(PropertyChangeListener listener) {
    changes.removePropertyChangeListener(listener);
  }



  /**
   * Alle Listener entfernen und Speicher freigebn.
   * 
   */
  public void removeAllListener() {
    for (PropertyChangeListener pl : changes.getPropertyChangeListeners()) {
      removeDifficultListener(pl);
    }
    for (ChangeListener cl : regler.getChangeListeners()) {
      regler.removeChangeListener(cl);
    }
    for (ActionListener al : check.getActionListeners()) {
      check.removeActionListener(al);
    }
  }



  /**
   * Wird gespielt oder nicht? Wenn das Spiel im Gange ist, dann muss der Zustand
   * {@code true} sein, damit keine Änderungen am Schwierigkeitsgrad vorgenommen
   * werden können.
   * 
   * @param value
   *              {@code true}, das Spiel ist im Gange
   */
//  void setStatus(Status status) {
//    this.status = status;
//  }

  @Override
  public void setLanguage(ISO639 code) {
    labelSchwiegrigkeitsgrad.setLanguage(code);
    check.setLanguage(code);
    multilingualSchwierigkeitsgrad.setLanguage(code);
    multilingualSchwierigkeitsgradAnpassen.setLanguage(code);
    multilingualColoringByConditions.setLanguage(code);
    multilingualColoringConditions.setLanguage(code);
    labMid.setLanguage(code);
    labMin.setLanguage(code);
    labMax.setLanguage(code);
    verloren.setLanguage(code);
    gewonnen.setLanguage(code);

  }



  @Override
  public void stateChanged(StatusEvent event) {
    status = event.hasStatus();

  }

}
