/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.BasicStroke;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import org.nexuswob.gui.JMouseButton;



/**
 * 
 * Ein Knopf mit einer Zahl. Die Klasse ist ein erweiterter JButton.
 * 
 */
public class JNumberButton extends JMouseButton {

  private static final long serialVersionUID = 6773018844630809643L;
  private String strRound;
  private int intRound;
  private BasicStroke stroke = new BasicStroke(3f);

  public JNumberButton(int round) {
    strRound = String.valueOf(round);
    intRound = round;
    activateHandCursor(true);
  }



  @Override
  public void paintComponent(Graphics g) {
    Graphics2D g2 = (Graphics2D) g;
    Stroke sOld = g2.getStroke();

    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_GASP);

    // Hintergrund Knopf
//    g2.setColor(Constants.OLIV);
//    g2.fillRect(0, 0, getWidth(), getHeight());

    // Gefüllter Kreis

    if (isPressed()) {
      g2.setColor(Constants.AS_GREY40);
      g2.fillOval(5, 5, 50, 50);
    }
    else {
      g2.setColor(Constants.AS_BLACK);
      g2.fillOval(5, 5, 50, 50);
    }

    // Weiße Umrandung
    g2.setStroke(stroke);
    g2.setColor(Constants.WHITE);
    g2.drawOval(5, 5, 50, 50);

    // Weißer Text
    g2.setStroke(sOld);
    if (intRound > 9) {
      g2.drawString(strRound, 23, 35);
    }
    else {
      g2.drawString(strRound, 27, 35);
    }
  }



  @Override
  public Dimension getPreferredSize() {
    return new Dimension(60, 60);
  }



  /**
   * Welche Runde?
   * 
   * @return die geklickte Spielrunde
   */
  public int getRound() {
    return intRound;
  }
}
