/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JPanel;

public class JPaintedFigurePanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3402063131880726709L;
	
	private Color color;

	public JPaintedFigurePanel(Color color) {
		this.color = color;
	}
	
	public void setFigureColor(Color color) {
		this.color = color;
		this.repaint();
	}
	
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		/*
		 * Der Rechteckhintergrund muss immer gezeichnet werden!
		 * Sonst entstehen ungewollte Pixel
		 */
		
		g2.setColor(Constants.OLIV);
		g2.fillRect(0, 0, this.getWidth(), this.getHeight());
		
		g2.setColor(color);
		g2.fillOval(0, 0, 50, 50);
	}
	
	@Override
	public Dimension getPreferredSize() {
		return(new Dimension(60, 60));
	}

}
