/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import javax.swing.JPanel;
import net.javacomm.mastermind.tool.DefColors;
import net.javacomm.mastermind.tool.Knopf;



@SuppressWarnings("serial")
public class JReportPanel extends JPanel {

//  private final static Logger log = LogManager.getLogger(JReportPanel.class);
  private String strReport;
  private EllipseFigure[] ellipse = new EllipseFigure[Knopf.values().length];

  public JReportPanel() {
    for (int index = 0; index < ellipse.length; index++) {
      ellipse[index] = new EllipseFigure(20 + 70 * index, 5);
    }
    strReport = String.valueOf(
        new char[] {DefColors.BLACK.charValue(), DefColors.BLACK.charValue(), DefColors.BLACK.charValue(),
            DefColors.BLACK.charValue()}
    );

    showReport();
  }



  /**
   * Der Rundenbericht wird gesetzt.
   * 
   * @param report
   */
  public void setReport(String report) {
    strReport = report;
  }



  @Override
  public void paintComponent(Graphics g) {
    Graphics2D g2 = (Graphics2D) g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setColor(Constants.OLIV);
    g2.fillRect(0, 0, getWidth(), getHeight());
    for (EllipseFigure element : ellipse) {
      element.paintGraphic(g2);
    }
  }



  /**
   * Der Rundenbericht wird angezeigt.
   */
  void showReport() {
    ellipse[Knopf.Knopf_1.intValue()]
        .setColor(DefColors.toDefColors(strReport.charAt(Knopf.Knopf_1.intValue())));
    ellipse[Knopf.Knopf_2.intValue()]
        .setColor(DefColors.toDefColors(strReport.charAt(Knopf.Knopf_2.intValue())));
    ellipse[Knopf.Knopf_3.intValue()]
        .setColor(DefColors.toDefColors(strReport.charAt(Knopf.Knopf_3.intValue())));
    ellipse[Knopf.Knopf_4.intValue()]
        .setColor(DefColors.toDefColors(strReport.charAt(Knopf.Knopf_4.intValue())));
    this.repaint();
  }

}
