/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.gui.manual;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import org.nexuswob.gui.JMouseButton;
import net.javacomm.mastermind.gui.Constants;



public class JManualControlPanel extends JPanel implements ActionListener {

  /**
   * 
   */
  private static final long serialVersionUID = -3274836476095464262L;

  private EtchedBorder etched = new EtchedBorder(EtchedBorder.LOWERED);
  private JMouseButton buttonNext = new JMouseButton(Constants.BUTTON_NEXT);
  private JMouseButton buttonBack = new JMouseButton(Constants.BUTTON_BACK);
  private JMouseButton buttonQuit = new JMouseButton(Constants.BUTTON_QUIT);
  private GridBagLayout gridbag = new GridBagLayout();
  private GridBagConstraints c = new GridBagConstraints();

  private PropertyChangeSupport change = new PropertyChangeSupport(this);

  public JManualControlPanel() {
    buttonNext.setBackground(Constants.OLIV);
    buttonNext.setForeground(Constants.WHITE);
    buttonNext.addActionListener(this);

    buttonBack.setBackground(Constants.OLIV);
    buttonBack.setForeground(Constants.WHITE);
    buttonBack.addActionListener(this);

    buttonQuit.setBackground(Constants.OLIV);
    buttonQuit.setForeground(Constants.WHITE);
    buttonQuit.addActionListener(this);

    setBackground(Constants.OLIV);
    setLayout(gridbag);

    c.gridy = 0;
    c.insets = new Insets(5, 25, 5, 25);

    c.gridx = 0;
    gridbag.setConstraints(buttonBack, c);
    this.add(buttonBack);

    c.gridx = 1;
    gridbag.setConstraints(buttonNext, c);
    this.add(buttonNext);

    c.gridx = 2;
    gridbag.setConstraints(buttonQuit, c);
    this.add(buttonQuit);

    setBorder(etched);
  }



  @Override
  public void actionPerformed(ActionEvent event) {
    String strEvent = event.getActionCommand();
    change.firePropertyChange(strEvent, -1, true);
  }



  @Override
  public synchronized void addPropertyChangeListener(PropertyChangeListener l) {

    if (change == null) {
      return;
    }

    change.addPropertyChangeListener(l);
  }



  @Override
  public synchronized void removePropertyChangeListener(PropertyChangeListener l) {

    if (change == null) {
      return;
    }

    change.removePropertyChangeListener(l);
  }
}
