/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.tool;

import java.util.Vector;



/******************************************************************/
/*                                                                */
/*                      Class CodeGenerator                       */
/*                                                                */
/******************************************************************/

/**
 * Der Codegenerator erzeugt zufällige Farbmuster.
 */
public final class CodeGenerator {

  public final static int BUTTONZAHL = 4;

  private CodeGenerator() {}



  /**
   * Ein Farbcode mit 4 Farben wird zurückgegeben. Gleiche Farben sind erlaubt.
   * 
   * 
   * @return ein Farbcode als String
   */
  public static String getColorCode() {
    StringBuffer colorCode = new StringBuffer();
    char color[] = {DefColors.YELLOW.charValue(), DefColors.ORANGE.charValue(), DefColors.RED.charValue(),
        DefColors.PURPUR.charValue(), DefColors.BLUE.charValue(), DefColors.GREEN.charValue()};
    int value;
    /* 4-stelligen Code erzeugen */

    for (int index = 0; index < BUTTONZAHL; index++) {
      value = Math.abs(RandomCode.getRandomNumeral() % color.length);
      colorCode.append(color[value]);
    }
    return colorCode.toString();
  }



  /**
   * Ein Farbcode mit 4 unterschiedlichen Farben wird zurückgegeben.
   * 
   * @return ein Farbcode als String
   */
  public static String getColorCodeUnique() {
    StringBuffer colorCode = new StringBuffer();
    char color[] = {DefColors.YELLOW.charValue(), DefColors.ORANGE.charValue(), DefColors.RED.charValue(),
        DefColors.PURPUR.charValue(), DefColors.BLUE.charValue(), DefColors.GREEN.charValue()};
    Vector<Character> colorVector = new Vector<Character>();

    // Vector füllen
    for (char character : color) {
      colorVector.add(character);
    }

    int value;
    /* 4-stelligen Code erzeugen, wo keine Farbe doppelt vorkommt */

    for (int index = 0; index < BUTTONZAHL; index++) {
      value = Math.abs(RandomCode.getRandomNumeral() % colorVector.size());
      colorCode.append(colorVector.elementAt(value));
      colorVector.remove(value);
    }
    return colorCode.toString();
  }
}