/*
 * The MIT License
 * Copyright © 2023 Thorsten Schulz <https://javacomm.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.javacomm.mastermind.tool;

import java.awt.Color;



/**
 * Alle Spielfarben.
 */
public enum DefColors {
  YELLOW(0, 'A', new Color(242, 225, 13)),
  ORANGE(1, 'B', new Color(241, 124, 37)),
  RED(2, 'C', new Color(220, 0, 0)),
  PURPUR(3, 'D', new Color(157, 6, 104)),
  BLUE(4, 'E', new Color(30, 44, 210)),
  GREEN(5, 'F', new Color(38, 159, 74)),
  BROWN(6, 'G', new Color(77, 55, 32)),
  WHITE(7, 'H', Color.WHITE),
  BLACK(8, 'I', Color.BLACK),
  GRAY(9, 'J', Color.GRAY);

  private int value;
  private char letter;
  private Color color;

  private DefColors(int value, char letter, Color color) {
    this.value = value;
    this.letter = letter;
    this.color = color;
  }



  /**
   * Konvertiere von {@code DefColors} nach {@code int}.
   * 
   * 
   * @return eine Farbe vom Typ {@code int}
   */
  public int intValue() {
    return value;
  }



  /**
   * Konvertiere von {@code DefColors} nach {@code char}.
   * 
   * 
   * 
   * @return eine Farbe vom Typ {@code char}
   */
  public char charValue() {
    return letter;
  }



  /**
   * Konvertiere von {@code DefColors} nach {@code Color}.
   * 
   * @return eine Farbe vom Typ {@code Color}
   */
  public Color colorValue() {
    return color;
  }



  /**
   * Konvertiere von {@code int} nach {@code DefColors}.
   * 
   * @param value
   *              ein RGB-Farbwert
   * 
   * @return eine Farbe vom Typ {@code DefColors}
   */
  public static DefColors toDefColors(int value) {

    for (DefColors tmp : DefColors.values()) {
      if (tmp.intValue() == value) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }



  /**
   * Konvertiere von {@code Color} nach {@code DefColors}.
   * 
   * @param value
   *              ein RGB-Farbwert
   * 
   * @return eine Farbe vom Typ {@code DefColors}
   */
  public static DefColors toDefColors(Color value) {

    for (DefColors tmp : DefColors.values()) {
      if (tmp.colorValue() == value) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }



  /**
   * Konvertiere von {@code char} nach {@code DefColors}.
   * 
   * @param value
   *              ein RGB-Farbwert
   * 
   * @return eine Farbe vom Typ {@code DefColors}
   */
  public static DefColors toDefColors(char value) {

    for (DefColors tmp : DefColors.values()) {
      if (tmp.charValue() == value) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }

}
