/**
 * Copyright © 2020 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.annotations.SerializedName;



/**
 * CHATMESSAGE ist eine gesendete Chatnachricht an alle User in einem Raum
 * 
 * @author llange
 *
 */
public class CHATMESSAGE extends MESSAGE implements Attachment {

  public final static String CHATUSER = "CHATUSER";
  public final static String ROOM = "ROOM";
  public final static String MESSAGE = "MESSAGE";
  public final static String DATETIME = "DATETIME";

  /* die Nachricht kommt aus diesem Raum */
  @SerializedName(value = "ROOM")
  private String room;

  /**
   * der Anwender hat gerade diese Nachricht geschrieben
   */
  @SerializedName(value = "MESSAGE")
  private String message;

  /**
   * der Anwender verschickt einen Anhang, ist optional
   */
  @SerializedName(value = "ATTACHMENT")
  private Long attachment;
  @SerializedName(value = "FILENAME")
  private String filename;
  @SerializedName(value = "FILESIZE")
  private Long filesize;

  // Chatuser enthält den Absender der Nachricht
  @SerializedName(value = "CHATUSER")
  private ChatUser chatuser;

  /**
   * In diesem Moment wurde die Nachricht in der Datenbank gespeichert.
   */
  @SerializedName(value = "DATETIME")
  private Long datetime;

  @Override
  public void setMessage(String value) {
    message = value;
  }



  @Override
  public String getMessage() {
    return message;
  }



  @Override
  public void setChatUser(ChatUser value) {
    chatuser = value;
  }



  @Override
  public ChatUser getChatUser() {
    return chatuser;
  }



  @Override
  public void setRoom(String value) {
    room = value;
  }



  @Override
  public String getRoom() {
    return room;
  }



  @Override
  public void setDatetime(Long value) {
    datetime = value;
  }



  @Override
  public Long getDatetime() {
    return datetime;
  }



  @Override
  public void setAttachment(Long value) {
    attachment = value;
  }



  @Override
  public void setFilename(String value) {
    filename = value;
  }



  @Override
  public void setFilesize(Long value) {
    filesize = value;
  }



  @Override
  public Long getAttachment() {
    return attachment;
  }



  @Override
  public String getFilename() {
    return filename;
  }



  @Override
  public Long getFilesize() {
    return filesize;
  }



  @Override
  public String toString() {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.registerTypeAdapter(CHATMESSAGE.class, new ChatMessageSerializer())
        .registerTypeAdapter(ChatUser.class, new ChatUserSerializer()).create();
    return gson.toJson(this);
  }

}
