/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;
import java.util.Arrays;
import java.util.Objects;



public class CONFERENCE extends MESSAGE {

  public final static String KONFERENZNAME = "KONFERENZNAME";
  public final static String ORGANISATOR_UID = "ORGANISATOR_UID";
  public final static String USERID = "USERID";
  public final static String SESSION = "SESSION";
  public final static String OFFHOOK = "OFFHOOK";
  public final static String KONFERENZRAUMUSER = "KONFERENZRAUMUSER";

  private String konferenzname;
  private String organisatorUid;
  private String userid;
  private String session;
  private Boolean offhook;
  private KonferenzraumUser[] konferenzraumUser;

  public KonferenzraumUser[] getKonferenzraumUser() {
    return konferenzraumUser;
  }



  public void setKonferenzraumUser(KonferenzraumUser[] konferenzraumUser) {
    this.konferenzraumUser = konferenzraumUser;
  }



  public String getKonferenzname() {
    return konferenzname;
  }



  public void setKonferenzname(String konferenzname) {
    this.konferenzname = konferenzname;
  }



  public String getOrganisatorUid() {
    return organisatorUid;
  }



  public void setOrganisatorUid(String organisatorUid) {
    this.organisatorUid = organisatorUid;
  }



  /**
   * Wer war der Absender dieser Botschaft?
   * 
   * @return dieser Anwender
   */
  public String getUserid() {
    return userid;
  }



  /**
   * Dieser Anwender sendet die Botschaft {@code CONFERENCE}.
   * 
   * @param userid
   *               dieser Anwender
   */
  public void setUserid(String userid) {
    this.userid = userid;
  }



  /**
   * Den Telefonförer auflegen oder abnehmen.
   * 
   * @return {@code true}, Hörer abnehmen; {@code false} Hörer aufgelegt
   */
  public Boolean isOffhook() {
    return offhook;
  }



  public void setOffhook(Boolean offhook) {
    this.offhook = offhook;
  }



  @Override
  public String toString() {
    Gson gson = new Gson();
    return gson.toJson(this);
  }



  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + Arrays.hashCode(konferenzraumUser);
    result = prime * result + Objects.hash(konferenzname, organisatorUid);
    return result;
  }



  @Override
  public boolean equals(Object obj) {
    if (this == obj) return true;
    if (obj == null) return false;
    if (getClass() != obj.getClass()) return false;
    CONFERENCE other = (CONFERENCE) obj;
    return Objects.equals(konferenzname, other.konferenzname)
        && Arrays.equals(konferenzraumUser, other.konferenzraumUser)
        && Objects.equals(organisatorUid, other.organisatorUid);
  }



  public String getSession() {
    return session;
  }



  public void setSession(String session) {
    this.session = session;
  }
}
