/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;



public class CONFERENCEAUDIO extends MESSAGE {

  public final static String USERID = "USERID";
  public final static String TARGET = "TARGET";
  public final static String CONTENT = "CONTENT";

  private String userid; // der Absender
  private String[] receiverSessions; // session empfänger
  private byte[] content; // inhalt ist base64 kodiert

  public CONFERENCEAUDIO() {
    super();
  }



  /**
   * Wer hat Nachricht gesendet?
   * 
   * @return dieser Anwender
   */
  public String getUserid() {
    return userid;
  }



  /**
   * Der Absender.
   * 
   * @param userid
   *               dieser Anwender
   */
  public void setUserid(String userid) {
    this.userid = userid;
  }



  public byte[] getContent() {
    return content;
  }



  public void setContent(byte[] content) {
    this.content = content;
  }



  @Override
  public String toString() {
    Gson gson = new Gson();
    return gson.toJson(this);
  }



  /**
   * Alle Empfängersessions.
   * 
   * @return alle Sessions
   */
  public String[] getReceiverSessions() {
    return receiverSessions;
  }



  /**
   * Der Audiostream wird an diese Empfänger gesendet.
   * 
   * @param receiverSessions
   *                         alle Empfängersessions
   */
  public void setReceiverSessions(String[] receiverSessions) {
    this.receiverSessions = receiverSessions;
  }
}
