/*
 * Copyright © 2025 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;



public class CONFERENCEMUTE extends MESSAGE {

  public final static String KONFERENZRAUM = "KONFERENZRAUM";
  public final static String ORGANISATOR = "ORGANISATOR";
  public final static String USERID = "USERID";
  public final static String RECEIVER_SESSIONS = "RECEIVER_SESSIONS";
  public final static String MUTE = "MUTE";

  private String konferenzraum;
  private String organisator; // ist eine userid
  private String userid; // der Absender
  private String[] receiverSessions;
  private Boolean mute; // true=mikro aus

  public CONFERENCEMUTE() {}



  public String getKonferenzraum() {
    return konferenzraum;
  }



  public String getOrganisator() {
    return organisator;
  }



  public String[] getReceiverSessions() {
    return receiverSessions;
  }



  public String getUserid() {
    return userid;
  }



  public Boolean isMute() {
    return mute;
  }



  /**
   * An diesen Konferenzraum wird gesendet.
   * 
   * @param konferenzraum
   *                      dieser Konferenzraum
   */
  public void setKonferenzraum(String konferenzraum) {
    this.konferenzraum = konferenzraum;
  }



  /**
   * Das Mikrofon wurde eingeschaltet oder ausgeschaltet.
   * 
   * @param mute
   *             {@code true}, stummgeschaltet
   */
  public void setMute(Boolean mute) {
    this.mute = mute;
  }



  /**
   * Der Konferenzraum wurde von dieser Person angelegt.
   * 
   * @param organisator
   *                    dieser EInrichter
   */
  public void setOrganisator(String organisator) {
    this.organisator = organisator;
  }



  /**
   * An diese Anwender wird gesendet.
   * 
   * @param receiverSessions
   *                         alle Empfänger
   */
  public void setReceiverSessions(String[] receiverSessions) {
    this.receiverSessions = receiverSessions;
  }



  /**
   * Dieser Anwender änderte seine Mikrofoneinstellung.
   * 
   * @param userid
   *               dieser Anwender
   */
  public void setUserid(String userid) {
    this.userid = userid;
  }



  @Override
  public String toString() {
    Gson gson = new Gson();
    return gson.toJson(this);
  }
}
