/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import java.lang.reflect.Type;



public class HistoryMessageDeserializer implements JsonDeserializer<HISTORYMESSAGE> {

  @Override
  public HISTORYMESSAGE deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {

    JsonObject jsonobject = json.getAsJsonObject();
    HEADER header = HEADER.toHeader(jsonobject.getAsJsonPrimitive(Protocol.HEADER).getAsString());
    JsonObject dataset = jsonobject.getAsJsonObject(Protocol.DATASET);
    String room = dataset.get(HISTORYMESSAGE.ROOM).getAsString();

    JsonArray record = dataset.getAsJsonArray(HISTORYMESSAGE.RECORD);
    int count = record.size();
    AttachmentImpl[] attachment = new AttachmentImpl[count];
    for (int index = 0; index < count; index++) {
      attachment[index] = new AttachmentImpl();

      JsonObject recordobject = record.get(index).getAsJsonObject();
      attachment[index].setDatetime(recordobject.getAsJsonPrimitive(RecordInterface.DATETIME).getAsLong());
      attachment[index].setMessage(recordobject.getAsJsonPrimitive(RecordInterface.MESSAGE).getAsString());
      attachment[index].setRoom(recordobject.getAsJsonPrimitive(RecordInterface.ROOM).getAsString());
      JsonObject chatuserobject = recordobject.getAsJsonObject(RecordInterface.CHATUSER);
      ChatUser chatuser = new ChatUser();
      chatuser.setBackgroundColor(chatuserobject.getAsJsonPrimitive(ChatUser.BACKGROUND_COLOR).getAsInt());
      chatuser.setForegroundColor(chatuserobject.getAsJsonPrimitive(ChatUser.FOREGROUND_COLOR).getAsInt());
      chatuser.setNickname(chatuserobject.getAsJsonPrimitive(ChatUser.NICKNAME).getAsString());
      chatuser.setUserid(chatuserobject.getAsJsonPrimitive(ChatUser.USERID).getAsString());
      attachment[index].setChatUser(chatuser);
      try {
        attachment[index].setFilename(recordobject.getAsJsonPrimitive(Attachment.FILENAME).getAsString());
        attachment[index].setFilesize(recordobject.getAsJsonPrimitive(Attachment.FILESIZE).getAsLong());
        attachment[index].setAttachment(recordobject.getAsJsonPrimitive(Attachment.ATTACHMENT).getAsLong());
      }
      catch (NullPointerException e) {}

    }

    HISTORYMESSAGE historymessage = new HISTORYMESSAGE();
    historymessage.setHeader(header);
    historymessage.setCommand(Command.HISTORYMESSAGE);
    historymessage.setDataset(Protocol.DATASET);
    historymessage.setRoom(room);
    historymessage.setAttachment(attachment);

    return historymessage;
  }

}
