/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;



public class HistoryMessageSerializer implements JsonSerializer<HISTORYMESSAGE> {

  @Override
  public JsonElement serialize(HISTORYMESSAGE src, Type typeOfSrc, JsonSerializationContext context) {

    JsonObject jsonObject = new JsonObject();
    JsonObject dataset = new JsonObject();
    JsonArray record = new JsonArray();
    JsonObject objectAttachment = new JsonObject();

    dataset.add("ROOM", context.serialize(src.getRoom()));

    Attachment[] attachment = src.getAttachment();
    if (attachment != null && attachment.length > 0) {
      record.add(objectAttachment);

      for (Attachment attach : attachment) {
        String room = attach.getRoom();
        Long datetime = attach.getDatetime();
        String message = attach.getMessage();
        Long anhang = attach.getAttachment();
        String filename = attach.getFilename();
        Long filesize = attach.getFilesize();
        ChatUser chatuser = attach.getChatUser();

        objectAttachment.add("ROOM", context.serialize(room));
        objectAttachment.add("DATETIME", context.serialize(datetime));
        objectAttachment.add("MESSAGE", context.serialize(message));
        objectAttachment.add("ATTACHMENT", context.serialize(anhang));
        objectAttachment.add("FILENAME", context.serialize(filename));
        objectAttachment.add("FILESIZE", context.serialize(filesize));
        objectAttachment.add("CHATUSER", context.serialize(chatuser));
      }
    }

    dataset.add("RECORD", context.serialize(record));
    jsonObject.add("HEADER", context.serialize(src.getHeader()));
    jsonObject.add("COMMAND", context.serialize(src.getCommand()));
    jsonObject.add("DATASET", context.serialize(dataset));
    return jsonObject;
  }

}
