/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;
import java.util.Objects;



public class KonferenzraumUser extends ChatUser {

  public final static String ANWESEND = "ANWESEND";
  public final static String SESSION = "SESSION";
  public final static String CHATUSER = "CHATUSER";
  public final static String MUTE = "MUTE";

  private boolean anwesend;
  private String session; // nur vorhanden, wenn der User anwesend ist
  private boolean mute; // standardmäßig ist das Mikrofon immer eingeschaltet.

  @Override
  public boolean equals(Object obj) {
    if (this == obj) return true;
    if (!super.equals(obj)) return false;
    if (getClass() != obj.getClass()) return false;
    KonferenzraumUser other = (KonferenzraumUser) obj;
    return anwesend == other.anwesend && Objects.equals(session, other.session);
  }



  /**
   * Der Server soll an diese Session senden.
   * 
   * @return diese Session gehört zu einem Anwender oder {@code null}, wenn der
   *         User nicht anwesend ist
   */
  public String getSession() {
    return session;
  }



  @Override
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + Objects.hash(anwesend, session);
    return result;
  }



  /**
   * Ist der Anwender in Telefonkonferenz anwesend oder abwesend?
   * 
   * @return {@code true}, der Anwender ist anwesend
   */
  public boolean isAnwesend() {
    return anwesend;
  }



  /**
   * Ist das Mikrofon stumm?
   * 
   * @return {@code true}, ist stumm und nimmt nichts auf
   */
  public boolean isMute() {
    return mute;
  }



  /**
   * 
   * 
   * @param online
   *               {@code true}, wenn der User in der Telefonkonferenz anwesend
   *               ist; {@code false} abwesend
   */
  public void setAnwesend(boolean online) {
    anwesend = online;
  }



  public void setMute(boolean mute) {
    this.mute = mute;
  }



  /**
   * Der Server soll später an diese Session senden.
   * 
   * @param session
   *                diese Session gehört zu einem Anwender
   */
  public void setSession(String session) {
    this.session = session;
  }



  @Override
  public String toString() {
    Gson gson = new Gson();
    return gson.toJson(this);
  }

}
