/**
 * Copyright © 2021 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

/**
 * Sprachkürzel de=Deutsch, en=Englisch, es=Spanisch
 * 
 * @author llange
 *
 */
public enum Language {

  de,
  en,
  es;

  /**
   * Konvertiere eine Sprachkürzel nach Language.
   * 
   * @param value
   *              ein Sprachkürzel wie de, en, es ...
   * 
   * @return das Sprachkürzel vom Typ Language
   * 
   * @throws wenn
   *              das übergebene Sprachkürzel ungültig ist
   */
  public static Language toLanguage(String value) {

    for (Language tmp : Language.values()) {
      if (tmp.toString().equals(value)) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }

}
