/**
 * Copyright © 2020 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.Gson;



public class ONHOOK extends MESSAGE {

  public final static String RECEIVER_USERID = "RECEIVER_USERID";
  public final static String CALLER_USERID = "CALLER_USERID";
  public final static String TO = "TO";
  public final static String SIGNAL = "SIGNAL";

  private String receiverUserid;
  private String callerUserid;
  private Boolean to;
  private boolean signal;

  /**
   * Wer hat aufgelegt?
   * 
   * @return {@code true}, der Caller hat aufgelegt
   */
  public Boolean isTo() {
    return to;
  }



  /**
   * Hat der Caller oder der Receiver aufgelegt?
   * 
   * @param to
   *           {@code true}, der Caller hat aufgelegt, {@code false} der Receiver
   *           hat aufgelegt
   */
  public void setTo(Boolean to) {
    this.to = to;
  }



  public String getReceiverUserid() {
    return receiverUserid;
  }



  /**
   * Der Empfänger.
   * 
   * @param selfUserid
   *                   dieser Anwender
   */
  public void setReceiverUserid(String selfUserid) {
    this.receiverUserid = selfUserid;
  }



  public String getCallerUserid() {
    return callerUserid;
  }



  /**
   * Der Anrufer.
   * 
   * @param callerUserid
   *                     dieser Anwender
   */
  public void setCallerUserid(String callerUserid) {
    this.callerUserid = callerUserid;
  }



  public boolean isSignal() {
    return signal;
  }



  /**
   * Busy tone einschalten oder aussschalten
   * 
   * @param signal
   *               {@code true}, busy tone soll abgespielt werden
   */
  public void setSignal(boolean signal) {
    this.signal = signal;
  }



  @Override
  public String toString() {
    Gson gson = new Gson();
    return gson.toJson(this);
  }

}
