/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import java.lang.reflect.Type;



public class SearchFilesDeserializer implements JsonDeserializer<SEARCHFILES> {

  @Override
  public SEARCHFILES deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {

    JsonObject jsonobject = json.getAsJsonObject();
    HEADER header = HEADER.toHeader(jsonobject.getAsJsonPrimitive(Protocol.HEADER).getAsString());
    JsonObject dataset = jsonobject.getAsJsonObject(Protocol.DATASET);
    String userid = null;
    try {
      userid = dataset.get(SEARCHFILES.USERID).getAsString();
    }
    catch (NullPointerException e) {}
    Boolean start = Boolean.TRUE;
    try {
      start = dataset.get(SEARCHFILES.START).getAsBoolean();
    }
    catch (NullPointerException e) {}

    JsonArray jsonArray = dataset.getAsJsonArray(SEARCHFILES.SEARCHFILE);
    Searchfile[] searchfile = new Searchfile[jsonArray.size()];
    int count = searchfile.length;
    for (int index = 0; index < count; index++) {
      JsonObject readObject = jsonArray.get(index).getAsJsonObject();
      searchfile[index] = new Searchfile();
      searchfile[index].setFilename(readObject.getAsJsonPrimitive(Searchfile.FILENAME).getAsString());
      searchfile[index].setIndex(readObject.getAsJsonPrimitive(Searchfile.INDEX).getAsInt());
      searchfile[index].setFilesize(readObject.getAsJsonPrimitive(Searchfile.FILESIZE).getAsLong());
    }

    SEARCHFILES searchfiles = new SEARCHFILES();
    searchfiles.setCommand(Command.SEARCHFILES);
    searchfiles.setHeader(header);
    searchfiles.setDataset(Protocol.DATASET);
    searchfiles.setUserid(userid);
    searchfiles.setStarted(start);
    searchfiles.setSearchfiles(searchfile);
    return searchfiles;
  }

}
