/*
 * Copyright © 2022 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.protocol;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;

public class SearchFilesSerializer implements JsonSerializer<SEARCHFILES> {

  @Override
  public JsonElement serialize(SEARCHFILES src, Type typeOfSrc, JsonSerializationContext context) {
    JsonObject jsonObject = new JsonObject();
    JsonObject dataset = new JsonObject();
    JsonArray serachfileArray = new JsonArray();
    if (src.getSearchfiles() != null) {
      for (Searchfile searchfiles : src.getSearchfiles()) {

        JsonObject file = new JsonObject();
        serachfileArray.add(file);

        file.add("INDEX", context.serialize(searchfiles.getIndex()));
        file.add("FILENAME", context.serialize(searchfiles.getFilename()));
        file.add("FILESIZE", context.serialize(searchfiles.getFilesize()));

      }
    }

    dataset.add("USERID", context.serialize(src.getUserid()));
    dataset.add("START", context.serialize(Boolean.TRUE));
    dataset.add("SEARCHFILE", context.serialize(serachfileArray));


    jsonObject.add("HEADER", context.serialize(src.getHeader()));
    jsonObject.add("COMMAND", context.serialize(src.getCommand()));
    jsonObject.add("DATASET", context.serialize(dataset));

    return jsonObject;
  }

}
