/**
 * Copyright © 2021 Luis Andrés Lange <http://javacomm.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.javacomm.transfer;

public enum Starttype {

  MANUAL_DE(1, "von Hand aus starten"),
  MANUAL_EN(2, "start manually"),
  MANUAL_ES(3, "iniciar manualmente"),
  AUTOMATIC_DE(4, "automatisch nach jedem Programmstart"),
  AUTOMATIC_EN(5, "automatically after each program start"),
  AUTOMATIC_ES(6, "automáticamente después de cada inicio del programa");


  private int modus;
  private String beschreibung;

  private Starttype(int modus, String beschreibung) {
    this.modus = modus;
    this.beschreibung = beschreibung;
  }



  public int getModus() {
    return modus;
  }



  @Override
  public String toString() {
    return beschreibung;
  }



  public static Starttype toTransferStarttype(int modus) {
    for (Starttype tmp : Starttype.values()) {
      if (tmp.getModus() == modus) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }



  public static Starttype toTransferStarttype(String beschreibung) {
    for (Starttype tmp : Starttype.values()) {
      if (tmp.toString().equals(beschreibung)) {
        return tmp;
      }
    }
    throw new IllegalArgumentException();
  }
}
